/* ***************************************************************************
   *                                                                         *
   *   Kindermann                        Wahlpflichtkurs Produktentwicklung  *
   *                                     Schuljahr: 2015-16                  *
   *                                                                         *
   *                                                                         *
   *                            Attiny85 ADC-Reader                          *
   *                                                                         *
   *                                                                         *
   ***************************************************************************/

#include "wx/wx.h"
#include "wx/image.h"
#include "wx/gdicmn.h"
#include <sstream>
#include <string>

#include "adc.h"

int tb_totalDevices, tbStatus, ti=0;
unsigned int scrx=20, scry=80, scrw=244, scrh=140;
float temp, old_temp;
unsigned int adcValue;
unsigned char rxBuffer[RX_BUFFER_SIZE];
unsigned char ROM_NO[8];

unsigned char	crc8;
int		LastDiscrepancy;
int 	LastFamilyDiscrepancy;
int 	LastDeviceFlag;

tinyboard *tb = NULL;

tinyboard* tinyboard_connect()
{
	tinyboard  *tempHandle = NULL;

	usb_init();
	usbOpenDevice(&tempHandle, VENDOR_ID, "*", PRODUCT_ID, "*", "*", NULL, NULL );

	return tempHandle;
}

void analog_init(tinyboard* tbHandle, unsigned char voltageRef)
{
	tbStatus=usb_control_msg(tbHandle, 0xC0, 35, (voltageRef<<8) | 0x07, 0, (char*) rxBuffer, 8, USB_TIMEOUT);
}

unsigned int analogRead(tinyboard* tbHandle, unsigned char channel)
{
	tbStatus=usb_control_msg(tbHandle, 0xC0, 15, channel, 0, (char*) rxBuffer, 8, USB_TIMEOUT);

	return ((rxBuffer[1] *256) + (rxBuffer[0]));
}

int usbGetStringAscii(usb_dev_handle *dev, int index, char *buf, int buflen)
{
char    buffer[256];
int     rval, i;

    if((rval = usb_get_string_simple(dev, index, buf, buflen)) >= 0) /* use libusb version if it works */
        return rval;
    if((rval = usb_control_msg(dev, USB_ENDPOINT_IN, USB_REQ_GET_DESCRIPTOR, (USB_DT_STRING << 8) + index, 0x0409, buffer, sizeof(buffer), 5000)) < 0)
        return rval;
    if(buffer[1] != USB_DT_STRING){
        *buf = 0;
        return 0;
    }
    if((unsigned char)buffer[0] < rval)
        rval = (unsigned char)buffer[0];
    rval /= 2;
    /* lossy conversion to ISO Latin1: */
    for(i=1;i<rval;i++){
        if(i > buflen)              /* destination buffer overflow */
            break;
        buf[i-1] = buffer[2 * i];
        if(buffer[2 * i + 1] != 0)  /* outside of ISO Latin1 range */
            buf[i-1] = '?';
    }
    buf[i-1] = 0;
    return i-1;
}


int usbOpenDevice(usb_dev_handle **device, int vendorID, char *vendorNamePattern, int productID, char *productNamePattern, char *serialNamePattern, FILE *printMatchingDevicesFp, FILE *warningsFp)
{
struct usb_bus      *bus;
struct usb_device   *dev;
usb_dev_handle      *handle = NULL;
int                 errorCode = USBOPEN_ERR_NOTFOUND;

    usb_find_busses();
    usb_find_devices();
    for(bus = usb_get_busses(); bus; bus = bus->next){
        for(dev = bus->devices; dev; dev = dev->next){  /* iterate over all devices on all busses */
            if((vendorID == 0 || dev->descriptor.idVendor == vendorID)
                        && (productID == 0 || dev->descriptor.idProduct == productID)){
                char    vendor[256], product[256], serial[256];
                int     len;
                handle = usb_open(dev); /* we need to open the device in order to query strings */
                if(!handle){
                    errorCode = USBOPEN_ERR_ACCESS;
                    if(warningsFp != NULL)
                        fprintf(warningsFp, "Warning: cannot open VID=0x%04x PID=0x%04x: %s\n", dev->descriptor.idVendor, dev->descriptor.idProduct, usb_strerror());
                    continue;
                }
                /* now check whether the names match: */
                len = vendor[0] = 0;
                if(dev->descriptor.iManufacturer > 0){
                    len = usbGetStringAscii(handle, dev->descriptor.iManufacturer, vendor, sizeof(vendor));
                }
                if(len < 0){
                    errorCode = USBOPEN_ERR_ACCESS;
                    if(warningsFp != NULL)
                        fprintf(warningsFp, "Warning: cannot query manufacturer for VID=0x%04x PID=0x%04x: %s\n", dev->descriptor.idVendor, dev->descriptor.idProduct, usb_strerror());
                }else{
                    errorCode = USBOPEN_ERR_NOTFOUND;
                    break;

                }
                usb_close(handle);
                handle = NULL;
            }
        }
        if(handle)  /* we have found a device */
            break;
    }
    if(handle != NULL){
        errorCode = 0;
        *device = handle;
    }
    if(printMatchingDevicesFp != NULL)  /* never return an error for listing only */
        errorCode = 0;
    return errorCode;
}


void delay(unsigned int duration)
{
	#ifdef LINUX
		usleep(duration*1000);
	#else
		Sleep(duration);
	#endif
}


int adcread()
{
	// -- Select ADC reference voltage
	// Those definetly needs calibration for precision measurement.
	analog_init(tb,VREF_VCC);
	// analog_init(tb,VREF_1100mV);
	// analog_init(tb,VREF_2560mV;

		adcValue=analogRead(tb, ADC_PIN2);

		// printf("> Value: %d\tVoltage: %f milivolts\n", adcValue,(float)((adcValue*(3000))/1024.0));
		// printf("> Value: %d\tVoltage: %f milivolts\n", adcValue,(float)((adcValue*(1100))/1024.0));
		// printf("> Value: %d\tVoltage: %f milivolts\n", adcValue,(float)((adcValue*(2560))/1024.0));

		if(tbStatus<0)
		{
			printf("> tbStatus: %d\n",tbStatus);
			printf("> Connection error!\n");
			return 0;
		}

		delay(100);

		return(adcValue);
}


enum
{
    wxTimer1 = 10,
    wxButton1 = 20,
    wxButton2 = 30
};


class MyFrame: public wxFrame
{
public:
    MyFrame(wxWindow *parent,	const wxWindowID id,
	    const wxString& title)
	  : wxFrame(parent, id, title)
    {
	SetIcon(wxICON(icon));

	m_pTimer = new wxTimer(this,wxTimer1);

	this->SetSizeHints( wxSize( 300,330 ), wxSize( 300,330 ) );

	m_statusBar1 = this->CreateStatusBar( 1, wxST_SIZEGRIP, wxID_ANY );
	wxFlexGridSizer* fgSizer1;
	fgSizer1 = new wxFlexGridSizer( 1, 1, 0, 0 );
	fgSizer1->SetFlexibleDirection( wxBOTH );
	fgSizer1->SetNonFlexibleGrowMode( wxFLEX_GROWMODE_SPECIFIED );


	fgSizer1->Add( 0, 10, 1, wxEXPAND, 5 );

	m_textCtrl1 = new wxTextCtrl( this, wxID_ANY, wxEmptyString, wxPoint( -1,40 ), wxSize( 246,45 ), wxTE_MULTILINE|wxTE_NO_VSCROLL|wxTE_READONLY );
	m_textCtrl1->SetMinSize( wxSize( 246,45 ) );
	m_textCtrl1->SetMaxSize( wxSize( 246,45 ) );

	fgSizer1->Add( m_textCtrl1, 1, wxALIGN_CENTER_HORIZONTAL, 5 );

	wxGridSizer* gSizer1;
	gSizer1 = new wxGridSizer( 2, 2, 0, 0 );

	gSizer1->SetMinSize( wxSize( 300,-1 ) );
	m_radioBtn1 = new wxRadioButton( this, wxID_ANY, wxT("Spannung"), wxDefaultPosition, wxDefaultSize, 0 );
	gSizer1->Add( m_radioBtn1, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5 );

	m_radioBtn2 = new wxRadioButton( this, wxID_ANY, wxT("10Bit-Wert"), wxDefaultPosition, wxDefaultSize, 0 );
	gSizer1->Add( m_radioBtn2, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5 );

	fgSizer1->Add( gSizer1, 0, wxEXPAND, 5 );


	fgSizer1->Add( 0, 148, 1, wxEXPAND, 5 );

	wxGridSizer* gSizer2;
	gSizer2 = new wxGridSizer( 2, 2, 0, 0 );

	gSizer2->SetMinSize( wxSize( 300,-1 ) );
	m_button1 = new wxButton( this, wxButton1, wxT("Start"), wxDefaultPosition, wxSize( 70,25 ), 0 );
	m_button1->SetMinSize( wxSize( 70,25 ) );

	gSizer2->Add( m_button1, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5 );

	m_button2 = new wxButton( this, wxButton2, wxT("Stop"), wxDefaultPosition, wxSize( 70,25 ), 0 );
	m_button2->Enable( false );
	m_button2->SetMinSize( wxSize( 70,25 ) );

	gSizer2->Add( m_button2, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5 );

	fgSizer1->Add( gSizer2, 1, wxEXPAND, 5 );

	this->SetSizer( fgSizer1 );
	this->Layout();

	this->Centre( wxBOTH );

    m_radioBtn1->SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));
    m_textCtrl1->Disable();
    m_radioBtn2->SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));
    m_radioBtn1->SetValue(true);

    wxSize s = GetClientSize();
    if((m_bitmap.GetWidth() != s.GetWidth()) ||
      (m_bitmap.GetHeight() != s.GetHeight())) {
      m_bitmap.Create(s.x,s.y);
      }

    wxMemoryDC dc;
    wxBrush brush(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE),
				  wxSOLID);
    dc.SelectObject(m_bitmap);
    dc.SetBackground(brush);
    dc.Clear();
    dc.SetBrush(*wxBLACK);
    dc.DrawRectangle(scrx,scry,scrw,scrh);

    m_pTimer->Start(100);
    }

    MyFrame()
    {}

    void Plot();
    void DrawOnBitmap();
    void OnTimerTimeout(wxTimerEvent& event);
    void OnButton1(wxCommandEvent &event);
    void OnButton2(wxCommandEvent &event);
    void OnPaint(wxPaintEvent& WXUNUSED(event));
    void CloseDlg(wxCloseEvent& event);
    DECLARE_EVENT_TABLE()

private:
    wxBitmap m_bitmap;
    wxTimer* m_pTimer;
    wxStatusBar* m_statusBar1;
	wxTextCtrl* m_textCtrl1;
	wxButton* m_button1;
	wxButton* m_button2;
	wxRadioButton* m_radioBtn1;
	wxRadioButton* m_radioBtn2;
};

 class MyApp:public wxApp
{
  bool OnInit()
  {
    if ( !wxApp::OnInit() )
    return false;

    MyFrame*  pFrame = new MyFrame(NULL,wxID_ANY,"ADC-Reader");
    pFrame->Show(true);

    return true;
  }
};


BEGIN_EVENT_TABLE(MyFrame, wxFrame)
    EVT_TIMER(wxTimer1,MyFrame::OnTimerTimeout)
    EVT_BUTTON(wxButton1, MyFrame::OnButton1)
    EVT_BUTTON(wxButton2, MyFrame::OnButton2)
    EVT_PAINT(MyFrame::OnPaint)
    EVT_CLOSE(MyFrame::CloseDlg)
END_EVENT_TABLE()
IMPLEMENT_APP(MyApp)


void MyFrame::DrawOnBitmap()
{
    wxMemoryDC dc;
    wxImage n_image = m_bitmap.ConvertToImage();
    m_bitmap = wxBitmap(n_image);
    dc.SelectObject(wxNullBitmap);
};


void MyFrame::OnTimerTimeout(wxTimerEvent& event)
    {
      wxFont* font = new wxFont();

      tb = tinyboard_connect();

      if(tb != NULL)
       {
       if (m_button2->IsEnabled())
         {
         temp = (float)(adcread()/1023.0f);

         m_textCtrl1->SetValue("");
         font->SetPointSize(25);
	     m_textCtrl1->SetFont(*font);

         if (m_radioBtn2->GetValue()) m_radioBtn1->SetValue(false);

         if (m_radioBtn1->GetValue())
           {
           wxString str = wxString::Format(wxT("      %0.3fV"), 3.6f*temp);
           m_textCtrl1->SetLabel(str);
           m_radioBtn2->SetValue(false);
           }
         else
           {
           wxString str = wxString::Format(wxT("        %0.0f"), 1023.0f*temp);
           m_textCtrl1->SetLabel(str);
           m_radioBtn2->SetValue(true);
           }

         wxMemoryDC dc;
         dc.SelectObject(m_bitmap);
         dc.SetPen(wxPen(wxColour(wxT("YELLOW")), 1, wxSOLID));
         dc.DrawLine(scrx+ti+1, scry, scrx+ti+1, scry+scrh);

         if (ti!=0)
           {
           dc.SetPen(wxPen(wxColour(*wxBLACK), 1, wxSOLID));
           dc.DrawLine(scrx+ti, scry, scrx+ti, scry+scrh);

           dc.SetPen(wxPen(wxColour(*wxRED), 1, wxSOLID));
           dc.DrawLine(scrx+ti-1, (float)(scry+((1.0f-old_temp) * 0.999f * scrh)), scrx+ti, (float)(scry+((1.0f-temp) * 0.999f * scrh)));
           }
         old_temp = temp;
         ti++;

         if (ti == scrw)
           {
           ti = 0;
           dc.SetPen(wxPen(wxColour(*wxBLACK), 1, wxSOLID));
           dc.DrawLine(scrx+scrw, scry, scrx+scrw, scry+scrh);

           dc.SetBrush(*wxBLACK);
           dc.DrawRectangle(scrx,scry,scrw,scrh);
           }
         Plot();
         }
       m_statusBar1->SetLabel("Digispark-Board gefunden !");
       }
      else
       {
       if (m_button2->IsEnabled())
         {
         m_textCtrl1->SetValue("");
         font->SetPointSize(12);
	     m_textCtrl1->SetFont(*font);
         m_textCtrl1->SetValue("          Digispark-Board\n           nicht gefunden!");
         }
       m_statusBar1->SetLabel("Digispark-Board nicht gefunden !");
       }
    }


void MyFrame::OnButton1(wxCommandEvent &WXUNUSED(event))
    {
    m_button1->Disable();
    m_button2->Enable();
    m_radioBtn1->Disable();
    m_radioBtn2->Disable();
    }


void MyFrame::OnButton2(wxCommandEvent &WXUNUSED(event))
    {
    m_button1->Enable();
    m_button2->Disable();
    m_radioBtn1->Enable();
    m_radioBtn2->Enable();
    }


void MyFrame::OnPaint(wxPaintEvent& WXUNUSED(event)) {

	   wxPaintDC dc(this);
       dc.DrawBitmap(m_bitmap,0,0,false);
       dc.SetBrush(*wxBLACK);
       dc.DrawRectangle(scrx,scry,scrw,scry);
    };


void MyFrame::Plot() {
	   wxClientDC dc(this);
	   DrawOnBitmap();
	   dc.DrawBitmap(m_bitmap,0,0,false);
    };


void MyFrame::CloseDlg(wxCloseEvent& event)
{
    m_pTimer->Stop();
    Destroy();
}
